<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class ExclusaoCardapio extends \PME\Services\TeknisaCrud {
   protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $environment;
	protected $entityManager;

    public function __construct($crudService, $parameterBag, $connection, $environment, $entityManager) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
	}
	
	public function excludePlate(DTO\Request\Row $request, DTO\Response $response) {
			$row = $request->getRow();
	    try {
	        $this->connection->beginTransaction();
	        $revenues = $this->hasRevenues($row);
	        
	        if ($revenues[0]['NRFAT'] == 0) {
	        		$cardia = $this->getCardia($row);
	        		$this->deleteCompPlates($cardia, $row);
        			$this->checkCardia($cardia, $row);
	        } else {
	            $response->addDataSet(new DataSet("exclude", array('message' => 'pme::possuiFaturamento', 'type' => 'error')));
	        }
	        
	        $this->connection->commit();
	        $response->addDataSet(new DataSet("exclude", array('message' => 'pme::excluidoComSucesso', 'type' => 'success')));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	// Verifica se cardápio já possui faturamento.
	private function hasRevenues($row) {
	    $sql = "SELECT NRFAT FROM (SELECT COUNT(*) AS NRFAT FROM EFETCLIEPME WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (CDTIPOCARD   = :P_CDTIPOCARD)   AND (CDFILIAL = :P_CDFILIAL)     AND (CDSERVFILI   = :P_CDSERVFILI)   AND (DTEFETCLPME BETWEEN :P_DTINICIAL AND :P_DTFINAL) AND (CDFILIFATU IS NOT NULL AND NRAPURACFAT IS NOT NULL))";
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD'],
            "P_CDFILIAL"     => $row['filterData']['aggregated'] !== null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_DTINICIAL"    => $row['initialDateExclude'],
            "P_DTFINAL"      => $row['finalDateExclude']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getCardia($row) {
      if ($row['unity'] != 'E') $sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, IDCARDESPEC FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND (INSTR(:P_CDFILIAL, CDFILIAL) <> 0 OR :P_CDFILIAL = 'T') AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL";
      else $sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, IDCARDESPEC FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND (INSTR(:P_CDFILIAL, CDFILIAL) <> 0 OR :P_CDFILIAL = 'T') AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL AND IDCARDESPEC = 'S'";
      
		$paramSelect = array(
         "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
         "P_NRCARDPME"    => $row['filterData']['NRCARDPME'],
         "P_CDFILIAL"     => $this->getCdFilial($row),
         "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
         "P_DTINICIAL"    => $row['initialDateExclude'],
         "P_DTFINAL"      => $row['finalDateExclude']
     );
     return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getCdFilial($row) {
		if ($row['unity'] === 'T') return $row['unity'];
		if ($row['unity'] === 'C') return $row['filterData']['cdfilcentpme'];
		
		if (count($row['selectSpecific']) == 0) return 'T';
		return implode("_", $row['selectSpecific']);
	}
	
	// Exclui a composição
	private function deleteCompPlates($cardia, $row) {
		$seqPlate = $this->concSeqPlate($row['platesSelect']);
		$query = "DELETE FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME AND (INSTR(:OLD_NRSEQPRACPME, NRSEQPRACPME) <> 0)";
		
		foreach ($cardia as $comp) {
			$params = array(
		      'P_CDFILCENTPME'	 => $comp['CDFILCENTPME'],
		      'P_NRCARDPME'  	 => $comp['NRCARDPME'],
		      'P_NRCARDIAPME'    => $comp['NRCARDIAPME'],
		      'OLD_NRSEQPRACPME' => $seqPlate
	   		);
	   	
	   		$this->connection->prepare($query)->execute($params);
		}
	}
	
	// Exclui cardápio da central e agregadas
	private function excludeCardia($card) {
		$query = "DELETE FROM CARDIAPME WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND NRCARDPME = :OLD_NRCARDPME AND NRCARDIAPME = :OLD_NRCARDIAPME";
		$params = array(
	      'OLD_CDFILCENTPME' => $card['CDFILCENTPME'],
	      'OLD_NRCARDPME'  	 => $card['NRCARDPME'],
	      'OLD_NRCARDIAPME'  => $card['NRCARDIAPME']
   	);
   	
   	$this->connection->prepare($query)->execute($params);
	}
	
	// Atualiza a cardadia
	private function updateCardia($cardia) {
		$query = "UPDATE CARDIAPME SET IDCARDESPEC = :P_IDCARDESPEC WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME";
		$params = array(
	      'P_CDFILCENTPME' => $cardia['CDFILCENTPME'],
	      'P_NRCARDPME'    => $cardia['NRCARDPME'],
	      'P_NRCARDIAPME'  => $cardia['NRCARDIAPME'],
	      'P_IDCARDESPEC'  => 'N'
   	);
	   
   	$this->connection->prepare($query)->execute($params);
	}
	
	private function concSeqPlate($plates) {
		$concPlates = '';
		foreach ($plates as $plate) {
			$explodeSeq = explode(" - ", $plate);
			$concPlates	.= $explodeSeq[1] . '_';
		}
		
		return $concPlates;
	}
	
	private function checkCardia($cardia, $row) {
		foreach ($cardia as $card) {
			$hasData = $this->hasDataCardia($card); // Busca composição do cardápio
	
			// Se não tiver composição: verifica T, C ou E
			if (count($hasData) == 0) {
				if ($row['unity'] === 'T') { // Opção de delete - Todos
					if ($card['IDCARDESPEC'] === 'S') $this->updateCardia($card);
					if ($card['CDFILCENTPME'] === $card['CDFILIAL']) $this->excludeAllCardia($card);
				}
				else if ($row['unity'] === 'C') { // Opção de delete - Central
					continue;
				}
				else if ($row['unity'] === 'E') { // Opção de delete - Específica
					$this->updateCardia($card);
				}
			}
			
			$this->callUpdateCentral($card, $row);
		}
	}
	
	// Verifica se ainda tem composição para a cardadia
	private function hasDataCardia($card) {
		$sql = "SELECT * FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME";
		
		$paramSelect = array(
         "P_CDFILCENTPME" => $card['CDFILCENTPME'],
         "P_NRCARDPME"    => $card['NRCARDPME'],
         "P_NRCARDIAPME"  => $card['NRCARDIAPME']
     );
      
      return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function deleteAggregated($card, $row) {
		$query = "DELETE FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL AND IDCARDESPEC = 'N'";
		$params = array(
	      'P_CDFILCENTPME' => $card['CDFILCENTPME'],
	      'P_NRCARDPME'    => $card['NRCARDPME'],
	      'P_CDSERVFILI'   => $card['CDSERVFILI'],
	      'P_DTINICIAL'    => $row['initialDateExclude'],
	      'P_DTFINAL'      => $row['finalDateExclude']
	   );
   	
   	$this->connection->prepare($query)->execute($params);
	}
	
	private function excludeAllCardia($card) {
		$query = "DELETE FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND DTCARDIAPME = :P_DTCARDIAPME";
		$params = array(
	      'P_CDFILCENTPME' => $card['CDFILCENTPME'],
	      'P_NRCARDPME'    => $card['NRCARDPME'],
	      'P_CDSERVFILI'   => $card['CDSERVFILI'],
	      'P_DTCARDIAPME'  => substr($card['DTCARDIAPME'], 0, 10)
	   );
   	
   	$this->connection->prepare($query)->execute($params);
	}
	
	private function callUpdateCentral($cardia, $row) {
		$comens = $this->getNewComen($cardia, $row);
		foreach ($comens as $comen) {
			$cardia['CDFILIAL'] = $comen['CDFILCENTPME'];
			$this->updateComenCentral($comen, $cardia);
		}
	}
	
	private function getNewComen($cardia, $row) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, CDSERVFILI, DTCARDIAPME, SUM(QTCOMESTCPME) AS QTCOMESTCPME, SUM(QTCLIESTCPME) QTCLIESTCPME, SUM(QTFUNESTCPME) AS QTFUNESTCPME, SUM(QTDOAESTCPME) QTDOAESTCPME FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL <> CDFILCENTPME AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL AND IDCARDESPEC = 'N' GROUP BY CDFILCENTPME, NRCARDPME, CDSERVFILI, DTCARDIAPME";
		
		$paramSelect = array(
         "P_CDFILCENTPME" => $cardia['CDFILCENTPME'],
         "P_NRCARDPME"    => $cardia['NRCARDPME'],
         "P_CDSERVFILI"   => $cardia['CDSERVFILI'],
         "P_DTINICIAL"    => $row['initialDateExclude'],
         "P_DTFINAL" 	  => $row['finalDateExclude']
     );
      
      return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function updateComenCentral($comen, $card) {
		$query = "UPDATE CARDIAPME SET QTCOMESTCPME = :P_QTCOMESTCPME, QTCLIESTCPME = :P_QTCLIESTCPME, QTFUNESTCPME = :P_QTFUNESTCPME, QTDOAESTCPME = :P_QTDOAESTCPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME = :P_DTCARDIAPME";
		
		$params = array(
	      'P_CDFILCENTPME' => $comen['CDFILCENTPME'],
	      'P_NRCARDPME'    => $comen['NRCARDPME'],
	      'P_CDSERVFILI'   => $comen['CDSERVFILI'],
	      'P_CDFILIAL'     => $card['CDFILIAL'],
	      'P_DTCARDIAPME'  => substr($comen['DTCARDIAPME'], 0, 10),
	      'P_QTCOMESTCPME' => $comen['QTCOMESTCPME'],
	      'P_QTCLIESTCPME' => $comen['QTCLIESTCPME'],
	      'P_QTFUNESTCPME' => $comen['QTFUNESTCPME'],
	      'P_QTDOAESTCPME' => $comen['QTDOAESTCPME'],
	   );
	   
   	$this->connection->prepare($query)->execute($params);
   }
   
   public function exclusaoDeCardapio(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
	    $this->connection->beginTransaction();
	    try {
	        $cardia = $this->getCardiaToExcludeCardapio($row);
	        
	        // if (empty($cardia)) throw new \Exception("CARDAPIO_NAO_EXISTE");
	        if (!empty($cardia)) {
	        	$this->deleteComp($cardia, $row);
				$this->deleteCardia($cardia, $row);
				
				$othersPlannedPlates = $this->getOthersPlannedPlates($cardia, $row);
				
				if (empty($othersPlannedPlates)) {
					$this->deleteCardapio($cardia, $row); 
				}
		        
		        $this->connection->commit();
		        // $response->addDataSet(new DataSet("exclude", array('message' => 'pme::excluidoComSucesso', 'type' => 'success')));
	        } else {
	        	$response->addDataSet(new DataSet("mensagem", array('message' => 'CARDAPIO_NAO_EXISTE', 'type' => 'error')));
	        }
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getCardiaToExcludeCardapio($row) {
	    $sql = "SELECT C.CDFILCENTPME, C.NRCARDPME, CD.NRCARDIAPME FROM CARDIAPME CD INNER JOIN CARDAPIOPME C  ON C.CDFILCENTPME  = CD.CDFILCENTPME AND C.NRCARDPME     = CD.NRCARDPME INNER JOIN TIPOCARD T     ON T.CDFILCENTPME  = C.CDFILCENTPME AND T.CDTIPOCARD    = C.CDTIPOCARD AND T.IDTPCARDATIV  = 'S' INNER JOIN PARAMFILIAL PF ON CD.CDFILCENTPME = PF.CDFILIAL, PARAMGERAL PG WHERE CD.CDFILCENTPME  = :P_CDFILCENTPME AND ((:P_CDAGRUPTPCARD = T.CDGRUPTIPOCARD) OR (:P_CDAGRUPTPCARD = 'T')) AND ((:P_CDSERVFILI = 'T') OR INSTR(:P_CDSERVFILI, CD.CDSERVFILI) > 0) AND ((:P_CDTIPOCARD = 'T') OR INSTR(:P_CDTIPOCARD, C.CDTIPOCARD ) > 0) AND CD.DTCARDIAPME >= TRUNC(DECODE(NVL(PG.IDUTILDTPROC, 'N'), 'N', SYSDATE, NVL(PF.DTPROCESSA,SYSDATE)) + NVL(PG.QTDIABLOQCAR, 0)) AND CD.DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL ORDER BY C.CDFILCENTPME, C.NRCARDPME, CD.NRCARDIAPME";
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['CDFILCENTPME'],
            "P_CDTIPOCARD"   => $row['CDTIPOCARD'],
            "P_CDSERVFILI"   => $row['CDSERVFILI'],
            "P_CDAGRUPTPCARD"=> $row['CDGRUPTIPOCARD'],
            "P_DTINICIAL"    => $row['DTDE'],
            "P_DTFINAL"      => $row['DTATE']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function deleteComp($cardia, $row) {
		$query = "DELETE FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME";
		
		foreach ($cardia as $comp) {
			$params = array(
		      'P_CDFILCENTPME'	 => $comp['CDFILCENTPME'],
		      'P_NRCARDPME'  	 => $comp['NRCARDPME'],
		      'P_NRCARDIAPME'    => $comp['NRCARDIAPME'],
	   		);
	   	
	   		$this->connection->prepare($query)->execute($params);
		}
	}
	
	private function deleteCardia($cardia, $row) {
		$query = "DELETE FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME";
		
		foreach ($cardia as $comp) {
			$params = array(
		      'P_CDFILCENTPME'	 => $comp['CDFILCENTPME'],
		      'P_NRCARDPME'  	 => $comp['NRCARDPME'],
		      'P_NRCARDIAPME'    => $comp['NRCARDIAPME'],
	   		);
	   	
	   		$this->connection->prepare($query)->execute($params);
		}
	}
	
	private function getOthersPlannedPlates($cardia, $row) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND INSTR(:P_NRCARDPME, NRCARDPME) <> 0";
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['CDFILCENTPME'],
            "P_NRCARDPME"    => implode('_', array_column($cardia, 'NRCARDPME')),
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function deleteCardapio($cardia, $row){
		$queryConf = "DELETE FROM CONFCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND INSTR(:P_NRCARDPME, NRCARDPME) <> 0"; 
		
		$paramsConf = array(
	      'P_CDFILCENTPME'	 => $row['CDFILCENTPME'], 
	      'P_NRCARDPME'  	 => implode('_', array_column($cardia, 'NRCARDPME')), 
   		);
   	
   		$this->connection->prepare($queryConf)->execute($paramsConf);
   		
   		$queryCard = "DELETE FROM CARDAPIOPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND INSTR(:P_NRCARDPME, NRCARDPME) <> 0"; 
		
		$paramsCard = array(
	      'P_CDFILCENTPME'	 => $row['CDFILCENTPME'], 
	      'P_NRCARDPME'  	 => implode('_', array_column($cardia, 'NRCARDPME')), 
   		);
   	
   		$this->connection->prepare($queryCard)->execute($paramsCard);
	}
}