<?php
namespace PME\Controllers\Servico;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ServicoController extends \PME\Services\TeknisaCrud {
  protected $crudService;
	protected $parameterBag;
	protected $environment;
	protected $dataSourceName;
	protected $connection;
	protected $errorCode = [
		"ORA-02292" => "ERROR_REGISTROFILHO"
	];

  public function __construct($crudService, $parameterBag, $connection, $environment) {
    parent::__construct($crudService, $parameterBag, $environment);
    $this->connection = $connection;
    $this->environment = $environment;
	}
	
	public function returnIdTipoServ($row){
	  $nrorg = $this->environment->getNrOrg();
	  $sql = "SELECT IDTIPOSERV FROM SERVICO WHERE IDTIPOSERV <> 'M' AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND NRORG = :P_NRORG";
	  $params = array(
	    "P_CDSERVFILI" => $row['CDSERVFILI'],
	    "P_CDFILIAL" => $row['CDFILIAL'],
	    "P_NRORG" => $nrorg
	  );
	  $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
		return $stmt->fetchAll();
	}
	
	public function saveServ(DTO\Request $request, DTO\Response $response) {
	  try {
      $rows = $request->getParameters()['rows'];
      $this->connection->beginTransaction();

      $queryInsert = "INSERT INTO SERVICO (CDSERVFILI, NMSERVICO, CDFILIAL, HRINICATEN, HRINICSERV, HRFINASERV, CDGRUPSERV, CDPRODUTO, IDTIPOSERV, IDSERVPRINC, IDSERVATIV, NRORG) 
                VALUES (:CDSERVFILI, :NMSERVICO, :CDFILIAL, :HRINICATEN, :HRINICSERV, :HRFINASERV, :CDGRUPSERV, :CDPRODUTO, :IDTIPOSERV, :IDSERVPRINC, :IDSERVATIV, :NRORG)";
      
      $queryUpdate = "UPDATE SERVICO SET NMSERVICO = :NMSERVICO, CDGRUPSERV = :CDGRUPSERV, HRINICATEN = :HRINICATEN, CDPRODUTO = :CDPRODUTO, 
                      HRINICSERV = :HRINICSERV, HRFINASERV = :HRFINASERV, IDTIPOSERV = :IDTIPOSERV, IDSERVPRINC = :IDSERVPRINC, IDSERVATIV = :IDSERVATIV
                      WHERE CDSERVFILI = :CDSERVFILI AND CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
      
      foreach($rows as $row) {
        $retorno = $this->returnIdTipoServ($row);
        if (!empty($retorno)) {
          if($retorno[0]['IDTIPOSERV'] == 'N'){
            $msg = array('message' => 'pme::servCadPlaIndustrial', 'type' => 'error');
          }
        } else {
          $params = array(
              'CDSERVFILI'  => $row['CDSERVFILI'],
              'NMSERVICO'   => $row['NMSERVICO'],
              'CDFILIAL'    => $row['CDFILIAL'],
              'HRINICATEN'  => $row['HRINICATEN'],
              'HRINICSERV'  => $row['HRINICSERV'],
              'HRFINASERV'  => $row['HRFINASERV'],
              'CDGRUPSERV'  => $row['CDGRUPSERV'],
              'CDPRODUTO'   => $row['CDPRODUTO'],
              'IDTIPOSERV'  => 'M',
              'IDSERVPRINC' => 'N',
              'IDSERVATIV'  => 'S',
              'NRORG' => $this->environment->getNrOrg()
            );
    
            if($row['__is_new'] === true) {
          	  $stmt = $this->connection->prepare($queryInsert);
          	} else {
          	  $stmt = $this->connection->prepare($queryUpdate);
          	}
          	
          	$stmt->execute($params);
          	$msg = array('message' => 'pme::salvoComSucesso', 'type' => 'success');
        }
      }
      $this->connection->commit();
      $response->addDataSet(new DataSet("result", $msg));
	  } catch (\Exception $e) {
	    $this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  public function deleteServ(DTO\Request $request, DTO\Response $response) {
    try {
      $parameters = $request->getParameters();
      $rows = $parameters['checkedRows'];
      $this->connection->beginTransaction();

      $query = "DELETE
                FROM SERVICO
               WHERE CDSERVFILI = :OLD_CDSERVFILI
                 AND CDFILIAL = :OLD_CDFILIAL
                 AND NRORG = :NRORG";
        
      foreach($rows as $row){
        $params = array(
          'OLD_CDSERVFILI' => $row['CDSERVFILI'],
          'OLD_CDFILIAL' => $row['CDFILIAL'],
          'NRORG' => $this->environment->getNrOrg()
        );
        $this->connection->prepare($query)->execute($params);
      }
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $errorMessage = $e->getMessage();
			foreach($this->errorCode as $key => $val){
				if(strpos($errorMessage, $key)) $errorMessage = $val;
			}
			$response->setError(new Error($errorMessage, 500));
  	}
  }

  public function saveConfigPratServ(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $rows = (Array) $request->getRow();
      
      $rowsToDelete = [];
      foreach($rows as $item) {
        if(!\Zeedhi\Framework\Util\Functions::arrayKeyExists('__is_new', $item) || $item['__is_new'] === false) {
          array_push($rowsToDelete, $item);
        }
      }
      if(!empty($rowsToDelete)) {
        $this->deleteConfigPrat($rowsToDelete); 
      }
      foreach($rows as &$row) {
        $row['__is_new'] = true;
      }

      $this->insertConfigPratServ($rows);
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", [true]));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  public function dadosGridConfigPrat(DTO\Request $request, DTO\Response $response) {
    try {
      $row = $request->getRow();
      
      $query = "SELECT CONFCARD.CDTIPOPRAT || CONFCARD.NRSEQCNFCARD AS KEY, CONFCARD.CDFILIAL, CONFCARD.CDSERVFILI, CONFCARD.NRSEQCNFCARD, CONFCARD.NRORDCNFCARD, CONFCARD.CDTIPOPRAT, 
                CONFCARD.IDOPCCNFCARD, CONFCARD.IDIMPCNFCARD, TIPOPRAT.NMTIPOPRAT, CONFCARD.NRORG FROM CONFCARD, TIPOPRAT 
              WHERE CONFCARD.CDFILIAL = :CDFILIAL AND CONFCARD.CDSERVFILI = :CDSERVFILI AND TIPOPRAT.CDTIPOPRAT = CONFCARD.CDTIPOPRAT ORDER BY CONFCARD.CDFILIAL, CONFCARD.CDSERVFILI, CONFCARD.NRORDCNFCARD";
  
      $params = array(
        'CDFILIAL' => $row['CDFILIAL'],
        'CDSERVFILI' => $row['CDSERVFILI']
      );
      $stmt = $this->connection->prepare($query);
  		$stmt->execute($params);
  		$data = $stmt->fetchAll();
  		$response->addDataSet(new DataSet("tipoprat", $data));
    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
  
  private function insertConfigPratServ($rows) {
    $query = "INSERT INTO CONFCARD (CDFILIAL, CDSERVFILI, NRSEQCNFCARD, NRORDCNFCARD, CDTIPOPRAT, IDOPCCNFCARD, IDIMPCNFCARD, NRORG, NRPAGCNFCARD) 
              VALUES (:CDFILIAL, :CDSERVFILI, :NRSEQCNFCARD, :NRORDCNFCARD, :CDTIPOPRAT, :IDOPCCNFCARD, :IDIMPCNFCARD, :NRORG, :NRPAGCNFCARD)";

    foreach($rows as $row) {
      
      $params = array(
        'CDFILIAL' => $row['CDFILIAL'],
        'CDSERVFILI' => $row['CDSERVFILI'],
        'NRSEQCNFCARD' => $row['NRSEQCNFCARD'],
        'NRORDCNFCARD' => $row['NRORDCNFCARD'],
        'CDTIPOPRAT' => $row['CDTIPOPRAT'],
        'IDOPCCNFCARD' => $row['IDOPCCNFCARD'],
        'IDIMPCNFCARD' => $row['IDIMPCNFCARD'],
        'NRORG' => $this->environment->getNrOrg(),
        'NRPAGCNFCARD' => $row['NRPAGCNFCARD']
      );
      $stmt = $this->connection->prepare($query);
      
      $stmt->execute($params);
    }
  }
  
  public function deleteConfigPrat($rows) {
    try {
      $query = "DELETE
                FROM CONFCARD
               WHERE CDSERVFILI = :OLD_CDSERVFILI
                 AND CDFILIAL = :OLD_CDFILIAL
                 AND NRORG = :NRORG";
      
      $params = array(
        'OLD_CDSERVFILI' => $rows[0]['CDSERVFILI'],
        'OLD_CDFILIAL' => $rows[0]['CDFILIAL'],
        'NRORG' => $rows[0]['NRORG'],
      );
      $stmt = $this->connection->prepare($query);
  	  $stmt->execute($params);
  	  
    } catch (\Exception $e) {
      throw new $e;
    }
  }
  
  public function exportService(DTO\Request $request, DTO\Response $response) {
    try {
      $row = $request->getRow();
      $servicos = explode("_", $row['SERVICO_EXPORT']);
     
      $this->connection->beginTransaction();
    
      $query = "INSERT INTO SERVICO (CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, HRINICATEN,
                  QTCOMESTISEG, QTCOMESTITER, QTCOMESTIQUA, QTCOMESTIQUI,
                  QTCOMESTISEX, QTCOMESTISAB, QTCOMESTIDOM, QTCOMESTIFER,
                  QTCLIESTISEG, QTCLIESTITER, QTCLIESTIQUA, QTCLIESTIQUI,
                  QTCLIESTISEX, QTCLIESTISAB, QTCLIESTIDOM, QTCLIESTIFER,
                  QTFUNESTISEG, QTFUNESTITER, QTFUNESTIQUA, QTFUNESTIQUI,
                  QTFUNESTISEX, QTFUNESTISAB, QTFUNESTIDOM, QTFUNESTIFER,
                  QTDOAESTISEG, QTDOAESTITER, QTDOAESTIQUA, QTDOAESTIQUI,
                  QTDOAESTISEX, QTDOAESTISAB, QTDOAESTIDOM, QTDOAESTIFER,
                  CDPRODUTO, IDFATUSERV, IDSERVPADR, IDTIPOSERV,
                  CDROTATRANS, HRINICSERV, HRFINASERV,
                  IDSERVTRAN, VRPESOCUSTO, IDCONSSERVCA, IDSERVPRINC,
                  IDUTRETIPRAT, IDTPSERVHOSP, CDSERPADDIE, IDSERVATIV)
          (SELECT  F.CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, HRINICATEN,
                  QTCOMESTISEG, QTCOMESTITER, QTCOMESTIQUA, QTCOMESTIQUI,
                  QTCOMESTISEX, QTCOMESTISAB, QTCOMESTIDOM, QTCOMESTIFER,
                  QTCLIESTISEG, QTCLIESTITER, QTCLIESTIQUA, QTCLIESTIQUI,
                  QTCLIESTISEX, QTCLIESTISAB, QTCLIESTIDOM, QTCLIESTIFER,
                  QTFUNESTISEG, QTFUNESTITER, QTFUNESTIQUA, QTFUNESTIQUI,
                  QTFUNESTISEX, QTFUNESTISAB, QTFUNESTIDOM, QTFUNESTIFER,
                  QTDOAESTISEG, QTDOAESTITER, QTDOAESTIQUA, QTDOAESTIQUI,
                  QTDOAESTISEX, QTDOAESTISAB, QTDOAESTIDOM, QTDOAESTIFER,
                  CDPRODUTO, IDFATUSERV, IDSERVPADR, IDTIPOSERV,
                  CDROTATRANS, HRINICSERV, HRFINASERV,
                  IDSERVTRAN, VRPESOCUSTO, IDCONSSERVCA, IDSERVPRINC,
                  IDUTRETIPRAT, IDTPSERVHOSP, CDSERPADDIE, IDSERVATIV
          FROM FILIAL F, (SELECT * FROM SERVICO
                          WHERE CDFILIAL =  :P_CDFILCENTPME
                          AND CDSERVFILI = :P_CDSERVFILI) SERV
          WHERE F.CDFILCENTPME = :P_CDFILCENTPME
            AND ((INSTR(:P_CDFILIAL, F.CDFILIAL) > 0) OR (:P_CDFILIAL= 'T'))
            AND NOT EXISTS(SELECT CDSERVFILI
                           FROM SERVICO S
                           WHERE F.CDFILIAL = S.CDFILIAL
                            AND S.CDSERVFILI = :P_CDSERVFILI))";
      
      foreach($servicos as $servico) {    
        $params = array(
          'P_CDFILCENTPME' => $row['UNIDADE_EXPORT'],
          'P_CDFILIAL'     => $row['UNIDADEAGREG'],
          'P_CDSERVFILI'   => $servico
        );
        $stmt = $this->connection->prepare($query);
    	  $stmt->execute($params);
      }
      $this->connection->commit();
	    $response->addDataSet(new DataSet("success", array(true)));
	  } catch (\Exception $e) {
	    	$this->connection->rollback();
			  $response->setError(new Error($e->getMessage(), 500));
	  }
  }
  
  public function deleteConfig(DTO\Request $request, DTO\Response $response) {
	    try {
          $parameters = $request->getParameters();
          $rows = $parameters['checkedRows'];
          
          $this->connection->beginTransaction();

          $query = "DELETE
                    FROM CONFCARD
                   WHERE CDSERVFILI = :OLD_CDSERVFILI
                     AND CDFILIAL = :OLD_CDFILIAL
                     AND NRSEQCNFCARD = :OLD_NRSEQCNFCARD
                     AND NRORG = :NRORG";
            // AND CDTIPOPRAT = :OLD_CDTIPOPRAT
          foreach($rows as $row){
            $params = array(
              'OLD_CDSERVFILI' => $row['CDSERVFILI'],
              'OLD_CDFILIAL' => $row['CDFILIAL'],
              // 'OLD_CDTIPOPRAT' => $row['CDTIPOPRAT'],
              'OLD_NRSEQCNFCARD' => $row['NRSEQCNFCARD'],
              'NRORG' => $row['NRORG'],
            );
	          $this->connection->prepare($query)->execute($params);
          }
	        $this->connection->commit();
	        $response->addDataSet(new DataSet("success", array(true)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
  }

  public function returnDadosPadraoServ(DTO\Request $request, DTO\Response $response) {
    $row = $request->getRow();
    $nrorg = $this->environment->getNrOrg();
    $query = "SELECT SE.CDSERVPADR, SE.NMSERVPADR, SE.CDPRODUTO, PR.CDARVPROD, PR.NMPRODUTO, SE.NRORG FROM SERVPADR SE, PRODUTO PR WHERE SE.CDPRODUTO = PR.CDPRODUTO AND SE.NRORG = :NRORG AND ROWNUM = 1 ORDER BY NLSSORT(SE.CDSERVPADR,'NLS_SORT=BINARY_AI')";
    $params = array(
      "NRORG" => $nrorg
    );
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
    $result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
  }
  
  public function saveAgregada(DTO\Request $request, DTO\Response $response) {
    $row = $request->getParameters()['rows'];
    
    if ($row[0]['__is_new']) {
      $query = "SELECT CDFILIAL, IDSERVINCFILAGR FROM PARAMPLANEJA WHERE CDFILIAL = :P_CDFILIAL";
      $params = array(
        "P_CDFILIAL" => $row[0]['CDFILIAL']
      );
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
      $result = $stmt->fetchAll();

      if ($result[0]['IDSERVINCFILAGR'] === 'S') {
        $queryInsert = "INSERT INTO SERVICO (CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, HRINICATEN, CDPRODUTO, IDFATUSERV, IDSERVPADR, IDTIPOSERV, CDROTATRANS, HRINICSERV, HRFINASERV, IDSERVTRAN, VRPESOCUSTO, IDCONSSERVCA, IDSERVPRINC, IDUTRETIPRAT, IDTPSERVHOSP, CDSERPADDIE, IDSERVATIV) (SELECT  F.CDFILIAL, CDSERVFILI, NMSERVICO, CDGRUPSERV, HRINICATEN, CDPRODUTO, IDFATUSERV, IDSERVPADR, IDTIPOSERV, CDROTATRANS, HRINICSERV, HRFINASERV, IDSERVTRAN, VRPESOCUSTO, IDCONSSERVCA, IDSERVPRINC, IDUTRETIPRAT, IDTPSERVHOSP, CDSERPADDIE, IDSERVATIV FROM FILIAL F, (SELECT * FROM SERVICO WHERE CDFILIAL =  :P_CDFILCENTPME AND CDSERVFILI = :P_CDSERVFILI) SERV WHERE F.CDFILCENTPME = :P_CDFILCENTPME AND NOT EXISTS(SELECT CDSERVFILI FROM SERVICO S WHERE S.CDFILIAL = F.CDFILIAL AND S.CDSERVFILI = :P_CDSERVFILI))";
        $params = array(
          "P_CDFILCENTPME" => $row[0]['CDFILCENTPME'],
          "P_CDSERVFILI"   => $row[0]['CDSERVFILI']
        );
        $stmt = $this->connection->prepare($queryInsert);
        $stmt->execute($params);
      }
    }
  }
}


