import {
  Select, Modal, ModalService, Radio, Button, Form, Number, SelectMultiple, SelectTree, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IEventParam, Loader, Http, I18n,
} from '@zeedhi/core';

export class QuantidadePorPessoaController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');
  
  private pratoController = Loader.getInstance('Pla01300PratoController');
  
  public modalCalcQuantidadePessoa!: Modal;
  
  public inicContainer() {
    // const project = localStorage.getItem('PRODUCT_ID');
    const cardapio = Metadata.getInstance<Select>('TPCARD');

    if (this.utilitiesPLA.isMerenda()) cardapio.isVisible = true;
    else cardapio.isVisible = false;
  }

  private async createModalCalcQuantidadePessoa() {
    this.modalCalcQuantidadePessoa = await ModalService.createFromJsonPost('',
      { jsonPath: 'planejamento/prato/modals/formCalcQuantidadePessoa' });
  }

  public closeModalCalcQuantidadePessoa() {
    this.modalCalcQuantidadePessoa.hide();
    this.modalCalcQuantidadePessoa.destroy();
    this.modalCalcQuantidadePessoa = this.utilities.modalEmpty;
  }
  
  public openModalCalcQuantidadePessoa() {
    if (!this.modalCalcQuantidadePessoa) {
      this.createModalCalcQuantidadePessoa().then(() => {
        this.modalCalcQuantidadePessoa.show();
      });
    } else {
      this.modalCalcQuantidadePessoa.show();
    }
  }
  
  public tratProduct() {
    const receita = Metadata.getInstance<SelectMultiple>('IDTIPORECEITA');
    
    if (this.pratoController.project === 'PAE') {
      receita.datasource.updateData([
        {
          name: I18n.translate('Padrão'),
          value: 'P',
        },
        {
          name: I18n.translate('Tipo de Cardápio'),
          value: 'T',
        },
        {
          name: I18n.translate('Local'),
          value: 'L',
        },
        {
          name: I18n.translate('Serviço'),
          value: 'S',
        },
      ]);
    } else {
      receita.datasource.updateData([
        {
          name: I18n.translate('Padrão'),
          value: 'P',
        },
        {
          name: I18n.translate('Local'),
          value: 'L',
        },
        {
          name: I18n.translate('Serviço'),
          value: 'S',
        },
      ]);
    }
  }
  
  // validar
  public setFilterTpCardapio() {
    const cardapio = Metadata.getInstance<Select>('TPCARD');
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    cardapio.datasource.filter = filter;
    cardapio.datasource.get();
  }
  
  public radioConfig({ component }: IEventParam<Radio>) {
    const valorFixo = Metadata.getInstance<Number>('VRQTDEPESSOA');
    const valorRadio = component.value;
    if (valorRadio === '0') {
      valorFixo.addValidation('required', { message: 'Campo obrigatório' });
      valorFixo.disabled = false;
    } else {
      valorFixo.removeValidation('required');
      valorFixo.disabled = true;
    }
  }
  
  public procedureQtdPessoa({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formFilterCalcPessoa');
    
    const arrReceitas = form.value.IDTIPORECEITA;
    const findRecPadrao = arrReceitas.find((column: any) => column === 'P');
    
    if (form.validate()) {
      const valorFixo = Metadata.getInstance<Number>('VRQTDEPESSOA');
      const opcao = Metadata.getInstance<Radio>('ORDENA');
      
      if ((opcao.value === '0' && valorFixo.value > '0') || opcao.value === '1') {
        const row = form.value;
        row.CDOPERADOR = localStorage.getItem('CDOPERADOR');
        row.CDFILCENTPME = this.appController.getCdCentral();
        row.IDTIPOSERV = this.pratoController.project === 'PAE' ? 'M' : 'N';
        
        if (findRecPadrao === 'P') {
          if (this.pratoController.operatorValid !== false) {
            LoadingService.show(' ');
            Http.post('/procedureQtdPessoa', { requestType: 'Row', row }).then(() => {
              this.utilities.showAlert('alert', 'Atualizado com sucesso!', 'success');
            }).finally(() => {
              LoadingService.hide();
            });
          } else {
            this.utilities.showAlert('Alert', 'Modificação em receita padrão depende de autorização.', 'warning');
            this.pratoController.openModalGroupOperador(component);
          }
        } else {
          LoadingService.show(' ');
          Http.post('/procedureQtdPessoa', { requestType: 'Row', row }).then(() => {
            this.utilities.showAlert('alert', 'Atualizado com sucesso!', 'success');
          }).catch((error) => {
            if (error.indexOf('Allowed memory size of') >= 0) {
              this.utilities.showAlert('alert',
                'Volume da dados excede tempo de resposta, favor redefinir as opções de filtro na tela.',
                'error');
            } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
        }
      } else {
        this.utilities.showAlert('error', 'Quantidade deve ser maior que zero.', 'red');
        LoadingService.hide();
      }
    }
  }
  
  public setFilterAgregAgrup({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    
    const conditions = [
      {
        name: 'P_CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value?.CDGRUPFILI ? agrupUnid.value.CDGRUPFILI : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    component.datasource.loading = true;
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const unidadeValue = (unidade.value.length !== 0) ? unidade.value.join('_') : 'T';
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value.CDGRUPFILI : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPOSERVICO');
    const grupServValue = grupserv.value ? grupserv.value.CDGRUPSERV : 'T';
    
    const idTipoServ = this.utilitiesPLA.isMerenda() ? '' : 'N';
    
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: idTipoServ,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public verifyPratIni({ component }: IEventParam<SelectTree>) {
    if (component.value !== null) {
      const pratoIni = Metadata.getInstance<SelectTree>('PRATO_INICIAL');
      if (this.utilities.returnPrat(pratoIni.value, component.value)) {
        this.utilities.showAlert('Alert', 'Prato Inicial não pode ser maior que o Prato Final.', 'error');
        component.setValue(null);
      }
    }
  }
}
